#pragma once
#include <vector>
#include <string>
#include <fstream>
#include <limits>
#include <iomanip>
#include <cmath>

#ifdef _MSC_VER 
// Visual studio is a bit specific here...
#include <limits>
#endif

///
/// Converts the argument to string in an OS-independent manner
/// The point here is that NaN and inf may be printed differently, this 
/// takes care of that.
///
template<class T>
void printToStream(std::ostream& out, const T& in) {
    bool isNan = !(in == in);

#ifdef _MSC_VER
    static_assert(std::is_same<T, double>::value || std::is_same<T, int>::value,
        "We currently only support double or int");

    bool isFinite = false;

    if (std::is_same < T, double>::value) {
        isFinite = std::isfinite(double(in));
    }
    else if (std::is_same<T, int>::value) {
        isFinite = in < std::numeric_limits<int>::max();
    }
#else
    bool isFinite = std::isfinite(in);
#endif

    if (isNan) { // in is NaN
        out << "NaN";
    }
    if (isFinite) {
        out << in;
    }
    else {
        out << "inf";
    }
}

///
/// Writes the contents of the vector 'data' to the textfile filename
/// The output format should be load-able in MATLAB and Numpy using the
/// load-command.
///
template<typename T>
void writeToFile(const std::string& filename,
         const T& data) {
    
    std::ofstream file(filename.c_str());
    // Set highest possible precision, this way we are sure we are
    file << std::setprecision(std::numeric_limits<long double>::digits10 + 1);

    // Loop over vector and write output to file
    for(int i = 0; i < data.size(); ++i) {
        printToStream(file, data[i]);
        file << " ";
    }
    file << std::endl;

    // File closes automatically at end of scope!
}
